package service

import (
	"errors"
	"fmt"
	"runtime"
	"runtime/debug"
	"smart_tunnel_edge/mqtt"
	"smart_tunnel_edge/util/logger"
	"strings"
	"sync"
	"time"
)

func InitMqtt() {
	MqttService = GetHandler()
	MqttService.SubscribeTopics()
	go MqttService.Handler()
}

var MqttService *MqttHandler

type MqttHandler struct {
	queue *mqtt.MlQueue
}

var _handlerOnce sync.Once
var _handlerSingle *MqttHandler

func GetHandler() *MqttHandler {
	_handlerOnce.Do(func() {
		_handlerSingle = &MqttHandler{
			queue: mqtt.NewQueue(10000),
		}
	})
	return _handlerSingle
}

func (o *MqttHandler) SubscribeTopics() {
	mqtt.GetMQTTMgr().Subscribe("smart_intersection/#", mqtt.AtLeastOnce, o.HandlerData)
}

func (o *MqttHandler) HandlerData(m mqtt.Message) {
	for {
		ok, cnt := o.queue.Put(&m)
		if ok {
			break
		} else {
			logger.Logger.Errorf("HandlerData:查询队列失败,队列消息数量:%d", cnt)
			runtime.Gosched()
		}
	}
}

func (o *MqttHandler) Handler() interface{} {
	defer func() {
		if err := recover(); err != nil {
			go GetHandler().Handler()
			logger.Logger.Errorf("MqttHandler.Handler:发生异常:%s", string(debug.Stack()))
		}
	}()
	for {
		msg, ok, quantity := o.queue.Get()
		if !ok {
			time.Sleep(10 * time.Millisecond)
			continue
		} else if quantity > 1000 {
			logger.Logger.Warnf("数据队列累积过多,请注意优化,当前队列条数:%d", quantity)
		}
		m, ok := msg.(*mqtt.Message)
		if !ok {
			continue
		}

		_, topic, err := parseTopic(m.Topic())
		if err != nil {
			logger.Logger.Errorf("parseTopic err")
			continue
		}
		switch topic {
		case TopicDeviceGateway:

		case TopicDeviceCamera:

		default:
			fmt.Println("我是主题：：：", topic)
		}
	}
}

func parseTopic(topic string) (string, string, error) {
	strList := strings.Split(topic, "/")
	if len(strList) < 4 {
		return "", "", errors.New("不支持的topic")
	}
	topic = strings.Join(strList[2:], "/")
	return strList[1], topic, nil
}

const (
	TopicDeviceGateway = "device/gateway"
	TopicDeviceCamera  = "device/camera"
	TopicDeviceScreens = "device/screens"
)
